// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "airplanemodepage.h"
#include "widgets/switchwidget.h"
#include "widgets/settingsgroup.h"
#include "widgets/contentwidget.h"
#include "window/utils.h"
#include "configwatcher.h"

#include <QVBoxLayout>
#include <QDBusConnection>

#include <DTipLabel>

DWIDGET_USE_NAMESPACE

using namespace dcc;
using namespace dcc::widgets;
using namespace DCC_NAMESPACE::network;
using namespace dde::network;

const QString SHOW_DOCK_KEY = QStringLiteral("enable");

AirplaneModepage::AirplaneModepage(QWidget *parent)
    : ContentWidget(parent)
    , m_airplaneModeSwitch(new SwitchWidget)
    , m_airplaneInter(new AirplanInter("com.deepin.daemon.AirplaneMode", "/com/deepin/daemon/AirplaneMode", QDBusConnection::systemBus(), this))
    , m_showInDockSwitch(new SwitchWidget)
{
    m_airplaneModeSwitch->setTitle(tr("Airplane Mode"));
    m_showInDockSwitch->setTitle(tr("Show in the Dock"));

    auto *switchGrp = new SettingsGroup;
    switchGrp->appendItem(m_airplaneModeSwitch);
    auto *tip = new DTipLabel(
        tr("Enabling the airplane mode turns off wireless network, personal hotspot and Bluetooth"),
        switchGrp);
    tip->setWordWrap(true);
    tip->setAlignment(Qt::AlignLeft);
    tip->setMargin(8);  // 设置tips与列表中文字左对齐

    auto *showInDockSwitchGrp = new SettingsGroup;
    showInDockSwitchGrp->appendItem(m_showInDockSwitch);

    auto *airplaneModeSwitchLayout = new QVBoxLayout;
    airplaneModeSwitchLayout->addWidget(switchGrp, 0);
    airplaneModeSwitchLayout->addWidget(tip, 0);
    airplaneModeSwitchLayout->setMargin(0);
    airplaneModeSwitchLayout->setSpacing(0);

    auto *scrollLayout = new QVBoxLayout;
    scrollLayout->addLayout(airplaneModeSwitchLayout, 0);
    scrollLayout->addWidget(showInDockSwitchGrp, 0);
    scrollLayout->addStretch(1);
    scrollLayout->setMargin(0);

    auto *contentWidget = new QWidget(this);
    contentWidget->setLayout(scrollLayout);

    setContentsMargins(ThirdPageContentsMargins);
    setContent(contentWidget);

    //初始化开关状态
    m_airplaneModeSwitch->setChecked(m_airplaneInter->enabled());
    connect(m_airplaneModeSwitch, &SwitchWidget::checkedChanged, this, [this](bool enabled){
        m_airplaneModeSwitch->setDisabled(true);
        AirplaneEnable(enabled);
    });
    //接受改变信号,防止在dock修改不同步
    connect(m_airplaneInter, &AirplanInter::EnabledChanged, m_airplaneModeSwitch, [this](bool checked){
        m_airplaneModeSwitch->setDisabled(false);
        m_airplaneModeSwitch->setChecked(checked);
    });

    ConfigWatcher *watcher = ConfigWatcher::instance();
    m_showInDockSwitch->setChecked(watcher->airplaneModeEnabled());
    connect(m_showInDockSwitch, &SwitchWidget::checkedChanged, this, [ watcher ](bool show){
        watcher->setAirplaneModeEnabled(show);
    });
    connect(watcher, &ConfigWatcher::airplaneModeEnabledChanged, this, [ this ](bool enabled) {
        m_showInDockSwitch->setChecked(enabled);
    });
}

void AirplaneModepage::AirplaneEnable(const bool enable) const
{
    m_airplaneInter->Enable(enable);
}
