// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef VPNPAGE_H
#define VPNPAGE_H

#include "interface/namespace.h"

#include <DListView>

#include <QPointer>

#include <NetworkManagerQt/VpnSetting>
#include <NetworkManagerQt/Connection>

class QFileDialog;

namespace dcc {
  class ContentWidget;
  namespace widgets {
    class SettingsGroup;
    class SwitchWidget;
    class LoadingNextPageWidget;
  }
}

namespace dde {
  namespace network {
    class VPNItem;
  }
}

using namespace dde::network;
using namespace dcc::widgets;
DWIDGET_USE_NAMESPACE;

class ConnectionVpnEditPage;
class VpnPage : public QWidget
{
    Q_OBJECT

public:
    explicit VpnPage(QWidget *parent = nullptr);
    ~VpnPage() override;
    void jumpPath(const QString &searchPath);

Q_SIGNALS:
    void requestNextPage(dcc::ContentWidget *const w) const;
    void requestVpnEnabled(const bool enabled) const;
    void requestFrameKeepAutoHide(const bool autoHide) const;

private:
    bool settingIsValid(VPNItem *item) const;

private Q_SLOTS:
    void refreshVpnList(const QList<VPNItem *>& vpns);
    void updateVpnItems(const QList<VPNItem *>& vpns);

    void onActiveConnsInfoChanged();

    void importVPN();
    void createVPN();
    void changeVpnId();
    void showEditPage(VPNItem *vpn, bool needCheck = false);

private:
    SwitchWidget *m_vpnSwitch;

    QPointer<ConnectionVpnEditPage> m_editPage;
    QString m_editingConnUuid;

    DListView *m_lvProfiles;
    QStandardItemModel *m_modelProfiles;
    QFileDialog *m_importFile;
    QString m_newConnectionPath;

    static const int UuidRole = Qt::UserRole + 100;
};

class VPNParametersChecker
{
public:
    VPNParametersChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    virtual ~VPNParametersChecker();

    virtual bool isValid() const;

    static VPNParametersChecker *createVpnChecker(const NetworkManager::Connection::Ptr &connection);

protected:
    NMStringMap data() const;
    NetworkManager::VpnSetting *setting() const;

private:
    NetworkManager::VpnSetting::Ptr m_vpnSetting;
};

class L2TPChecker : public VPNParametersChecker
{
public:
    L2TPChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class PPTPChecker : public VPNParametersChecker
{
public:
    PPTPChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class VPNCChecker : public VPNParametersChecker
{
public:
    VPNCChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class OpenVPNChecker : public VPNParametersChecker
{
public:
    OpenVPNChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;

private:
    bool tlsIsValid() const;
    bool passIsValid() const;
    bool passTlsValid() const;
    bool staticKeyIsValid() const;
};

class StrongSwanChecker : public VPNParametersChecker
{
public:
    StrongSwanChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class OpenConnectChecker : public VPNParametersChecker
{
public:
    OpenConnectChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class SSTPChecker : public VPNParametersChecker
{
public:
    SSTPChecker(const NetworkManager::VpnSetting::Ptr &vpnSetting);
    bool isValid() const override;
};

class DefaultChecker : public VPNParametersChecker
{
public:
    DefaultChecker(bool defaultValue = false);
    bool isValid() const override;

private:
    bool m_defaultValue;
};

#endif // VPNPAGE_H
