// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef NETWORKPLUGIN_H
#define NETWORKPLUGIN_H

#include "pluginsiteminterface.h"

#include <QSharedPointer>

#define NETWORK_KEY "network-item-key"

class QDBusMessage;

namespace dde {
namespace network {
class NetManager;
class NetView;
class NetStatus;

class NetworkPlugin : public QObject, PluginsItemInterface
{
    Q_OBJECT

    Q_INTERFACES(PluginsItemInterface)
    Q_PLUGIN_METADATA(IID "com.deepin.dock.PluginsItemInterface" FILE "network.json")

public Q_SLOTS:
    void showNetworkDialog();
    void updateLockScreenStatus(bool visible);
    void updateIconColor();

public:
    explicit NetworkPlugin(QObject *parent = Q_NULLPTR);
    ~NetworkPlugin() Q_DECL_OVERRIDE;

    const QString pluginName() const override;
    const QString pluginDisplayName() const override;
    void init(PluginProxyInterface *proxyInter) override;
    void invokedMenuItem(const QString &itemKey, const QString &menuId, bool checked) override;
    void refreshIcon(const QString &itemKey) override;
    void pluginStateSwitched() override;
    bool pluginIsAllowDisable() override;
    bool pluginIsDisable() override;
    const QString itemCommand(const QString &itemKey) override;
    const QString itemContextMenu(const QString &itemKey) override;
    QWidget *itemWidget(const QString &itemKey) override;
    QWidget *itemTipsWidget(const QString &itemKey) override;
    QWidget *itemPopupApplet(const QString &itemKey) override;

    int itemSortKey(const QString &itemKey) override;
    void setSortKey(const QString &itemKey, int order) override;

    void pluginSettingsChanged() override;

private:
    void loadPlugin();
    void refreshPluginItemsVisible();
    bool eventFilter(QObject *watched, QEvent *event) override;

private:
    QPointer<QWidget> m_trayIcon;
    QPointer<QWidget> m_tipsWidget;
    dde::network::NetManager *m_manager;
    dde::network::NetView *m_netView;
    dde::network::NetStatus *m_netStatus;
    bool m_isLockScreen;
};
} // namespace network
} // namespace dde
#endif // NETWORKPLUGIN_H
