// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#ifndef NETSTATUS_H
#define NETSTATUS_H

#include <QIcon>
#include <QLabel>
#include <QObject>
#include <QPointer>

class QLabel;

namespace dde {
namespace network {

class NetManager;
class NetItem;
class NetWirelessItem;

//  网络状态，仅界面无业务
class NetStatus : public QObject
{
    Q_OBJECT
public:
    explicit NetStatus(NetManager *manager, QObject *parent = nullptr);
    ~NetStatus() override;

    enum class NetworkStatus {
        Unknown = 0,
        Disabled,
        Connected,
        Disconnected,
        Connecting,
        Failed,
        ConnectNoInternet,
        WirelessDisabled,
        WiredDisabled,
        WirelessConnected,
        WiredConnected,
        WirelessDisconnected,
        WiredDisconnected,
        WirelessConnecting,
        WiredConnecting,
        WirelessConnectNoInternet,
        WiredConnectNoInternet,
        WirelessFailed,
        WiredFailed,
        WiredIpConflicted,
        WirelessIpConflicted,
        Nocable,

        Count
    };
    Q_ENUM(NetworkStatus)

    void invokeMenuItem(const QString &menuId);
    bool needShowControlCenter() const;
    const QString contextMenu(bool hasSetting) const;
    QWidget *createItemTips() const;
    QWidget *createIconWidget() const;
    void refreshIcon();

    bool hasDevice() const;
    QString networkTips() const;
    QIcon networkIcon() const;

    inline NetworkStatus networkStatus() const { return m_networkStatus; }

protected Q_SLOTS:
    void onChildAdded(const NetItem *child);
    void onChildRemoved(const NetItem *child);
    void updateStatus();
    void updateNetworkTips();
    void updateNetworkIcon();
    void updateIconStr(const QString &iconstr);
    void nextAnimation();

Q_SIGNALS:
    void networkStatusChanged(NetworkStatus networkStatus);
    void networkTipsChanged(const QString &networkTips);
    void networkIconChanged(const QIcon &icon);
    void hasDeviceChanged(bool hasDevice);

private:
    NetWirelessItem *getConnectWirelessItem();

private:
    NetManager *m_manager;
    QString m_networkTips;
    NetworkStatus m_networkStatus;
    bool m_hasDevice;
    unsigned m_deviceFlag;

    QPointer<NetWirelessItem> m_activeItem;
    QString m_networkIconStr;
    QIcon m_networkIcon;

    QString m_iphtml;

    QStringList m_animationIcon;
    int m_currentAnimation;
    QTimer *m_animationTimer;
};

} // namespace network
} // namespace dde

#endif // NETSTATUS_H
