// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#ifndef NETDELEGATE_H
#define NETDELEGATE_H

#include <DWidget>

#include <QStyledItemDelegate>
class QSortFilterProxyModel;
class QVBoxLayout;

namespace Dtk {
namespace Widget {
class DSpinner;
class DSwitchButton;
} // namespace Widget
} // namespace Dtk

namespace dde {
namespace network {

class NetIconButton;
class NetItem;
class NetDeviceItem;
class NetWiredDeviceItem;
class NetWirelessDeviceItem;
class NetWiredItem;
class NetWirelessHiddenItem;
class NetWirelessItem;
enum class NetConnectionStatus;

class NetDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    explicit NetDelegate(const QSortFilterProxyModel *model, QObject *parent = nullptr);
    ~NetDelegate() override;

    // painting
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;

    // editing
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override;

    void showPassword(const QString &id, const QVariantMap &param);
    void showError(const QString &id, const QVariantMap &param);
    void updateInputValid(const QString &id, const QVariantMap &param);
    void closeInput();

Q_SIGNALS:
    void requestSetEnabled(const QString &id, bool enabled);
    void requestScan(const QString &id);
    void requestDisconnect(const QString &id);
    void requestConnect(const QString &id, const QVariantMap &param);
    void requestCheckInput(const QString &id, const QVariantMap &param);

    void requestShowPassword(const QString &id, const QVariantMap &param);
    void requestShowError(const QString &id, const QVariantMap &param);
    void requestUpdateInputValid(const QString &id, const QVariantMap &param);
    void requestCloseInput();

    void requestUpdateLayout();
    void requestShow(const QString &id);

private:
    const QSortFilterProxyModel *m_model;
};

class NetWidget : public QWidget
{
    Q_OBJECT
public:
    explicit NetWidget(NetItem *item, QWidget *parent = nullptr);
    ~NetWidget() Q_DECL_OVERRIDE;

    void setCentralWidget(QWidget *widget);

    void addPasswordWidget(QWidget *widget);
    void setNoMousePropagation(bool noMousePropagation);
    void removePasswordWidget();

Q_SIGNALS:
    void requestConnect(const QString &id, const QVariantMap &param);
    void requestCheckInput(const QString &id, const QVariantMap &param);

public Q_SLOTS:
    void showPassword(const QString &id, const QVariantMap &param);
    void showError(const QString &id, const QVariantMap &param);
    void onRequestCheckInput(const QVariantMap &param);
    void updateInputValid(const QString &id, const QVariantMap &param);
    void onSubmit(const QVariantMap &param);
    void closeInput();

protected:
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;

    inline NetItem *item() const { return m_item; }

private:
    NetItem *m_item;
    QVBoxLayout *m_mainLayout;
    bool m_noMousePropagation;
};

class NetDeviceWidget : public NetWidget
{
    Q_OBJECT
public:
    NetDeviceWidget(NetDeviceItem *item, QWidget *parent = nullptr);
    ~NetDeviceWidget() Q_DECL_OVERRIDE;

Q_SIGNALS:
    void requestSetEnabled(const QString &id, bool enabled);
    void requestScan(const QString &id);

protected Q_SLOTS:
    void onEnabledChanged(bool enabled);
    void onCheckedChanged(bool checked);
    void onScanClicked();

private:
    Dtk::Widget::DSwitchButton *m_switchBut;
};

class NetWirelessWidget : public NetWidget
{
    Q_OBJECT
public:
    NetWirelessWidget(NetWirelessItem *item, QWidget *parent = nullptr);
    ~NetWirelessWidget() Q_DECL_OVERRIDE;

Q_SIGNALS:
    void requestDisconnect(const QString &id);

public Q_SLOTS:
    void onStrengthChanged(int strength);
    void onSecureChanged(bool secured);
    void onStatusChanged(dde::network::NetConnectionStatus status);
    void onDisconnectClicked();

private:
    bool m_isWifi6;
    NetIconButton *m_secureBut;
    NetIconButton *m_iconBut;
    NetIconButton *m_connBut;
    Dtk::Widget::DSpinner *m_loading;
};

class NetWirelessHiddenWidget : public NetWidget
{
    Q_OBJECT
public:
    NetWirelessHiddenWidget(NetWirelessHiddenItem *item, QWidget *parent = nullptr);
    ~NetWirelessHiddenWidget() Q_DECL_OVERRIDE;
};

class NetWiredWidget : public NetWidget
{
    Q_OBJECT

public:
    NetWiredWidget(NetWiredItem *item, QWidget *parent = nullptr);
    ~NetWiredWidget() Q_DECL_OVERRIDE;

Q_SIGNALS:
    void requestDisconnect(const QString &id);

public Q_SLOTS:
    void onStatusChanged(NetConnectionStatus status);
    void onDisconnectClicked();

private:
    NetIconButton *m_iconBut;
    NetIconButton *m_connBut;
    Dtk::Widget::DSpinner *m_loading;
};

} // namespace network
} // namespace dde

#endif // NETDELEGATE_H
