// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CONFIGSETTING_H
#define CONFIGSETTING_H

#include <QObject>

namespace dde {
namespace network {

class ConfigSetting : public QObject
{
    Q_OBJECT

public:
    static ConfigSetting *instance();
    void alawaysLoadFromNM();               // 始终从NM中加载数据
    bool serviceFromNetworkManager() const; // 来源于NetworkManager的接口，如果是false，则从后端读取数据

    QStringList networkCheckerUrls() const; // 网络检测地址，用于检测网络连通性
    bool enableConnectivity() const;        // 是否开启或者禁用前端检测网络连通性的功能
    bool checkPortal() const;               // 是否检测网络认证信息
    int connectivityCheckInterval() const;  // 网络连通性检测时间间隔
    int wirelessScanInterval() const;       // 无线扫描间隔(ms)
    QString wpaEapAuthen() const;           // 企业网EAP认证方式
    QString wpaEapAuthmethod() const;       // 企业网内部认证方式

signals:
    void checkUrlsChanged(const QStringList &);
    void enableConnectivityChanged(bool);
    void checkPortalChanged(bool);
    void connectivityCheckIntervalChanged(int);
    void wirelessScanIntervalChanged(int);
    void wpaEapAuthenChanged(const QString &);
    void wpaEapAuthmethodChanged(const QString &);

private:
    explicit ConfigSetting(QObject *parent = nullptr);
    ~ConfigSetting();

private slots:
    void onValueChanged(const QString &key);

private:
    bool m_alwaysFromNM;
    bool m_loadServiceFromNM;
    bool m_enableConnectivity;
    bool m_checkPortal;
    int m_connectivityCheckInterval;
    int m_wirelessScanInterval;
    QStringList m_networkUrls;
    QString m_wpaEapAuthen;
    QString m_wpaEapAuthmethod;
};

} // namespace network
} // namespace dde

#endif // CONFIGSETTING_H
