// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CONNECTIVITYCHECKER_H
#define CONNECTIVITYCHECKER_H

#include <QObject>

#include "networkconst.h"

class QTimer;

namespace dde {

namespace network {

class NetworkDeviceBase;
class NetworkProcesser;
class AccessPoints;
class ConnectivityDetection;

// 网络连通性的检测
class ConnectivityChecker : public QObject
{
    Q_OBJECT

public:
    ConnectivityChecker(QObject *parent = Q_NULLPTR);
    virtual ~ConnectivityChecker();

    virtual void setCheckPortal(bool checkPortal);
    virtual Connectivity connectivity() const = 0;

signals:
    void connectivityChanged(const Connectivity &);
};

class LocalConnectionvityChecker : public ConnectivityChecker
{
    Q_OBJECT

public:
    LocalConnectionvityChecker(NetworkProcesser *networkProcesser, QObject *parent = Q_NULLPTR);
    ~LocalConnectionvityChecker() override;

    void setCheckPortal(bool checkPortal) override;
    Connectivity connectivity() const override;

private:
    void initDeviceConnect(const QList<NetworkDeviceBase *> &devices);
    void createConnectivityChecker();
    void initConnectivityChecker();
    void checkUnConnection();
    void setConnectivity(const Connectivity &connectivity);

private slots:
    void onCheckStatusChanged(bool isConnected);
    void onUpdateAccessPointConnect(const QList<AccessPoints *> accessPoints);
    void onDeviceStatusChanged(const DeviceStatus &deviceStatus);

private:
    NetworkProcesser *m_networkProcesser;
    ConnectivityDetection *m_connectivityChecker;
    QThread *m_checkThread;
    QTimer *m_checkTimer;
    QTimer *m_timer;
    QList<QMetaObject::Connection> m_checkerConnection;
    bool m_checkPortal;
    Connectivity m_connectivity;
};

class NMConnectionvityChecker : public ConnectivityChecker
{
    Q_OBJECT

public:
    NMConnectionvityChecker(NetworkProcesser *networkProcesser, QObject *parent = Q_NULLPTR);
    ~NMConnectionvityChecker() override;
    Connectivity connectivity() const override;

private:
    NetworkProcesser *m_networkProcesser;
};

class ConnectivityDetection : public QObject
{
    Q_OBJECT

public:
    explicit ConnectivityDetection(QObject *parent = nullptr);

Q_SIGNALS:
    void checkFinished(bool connectivity) const;

public Q_SLOTS:
    void startCheck();

private Q_SLOTS:
    void onUpdateUrls(const QStringList &urls);


private:
    QStringList m_checkUrls;
    bool m_isRunning;
};

}   // namespace network

}   // namespace dde

#endif // CONNECTIVITYCHECKER_H
