// SPDX-FileCopyrightText: 2018 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CONNECTIVITYPROCESSER_H
#define CONNECTIVITYPROCESSER_H

#include "networkconst.h"

#include <QObject>

class QTimer;
class QThread;

namespace dde {

namespace network {

class NetworkProcesser;
class ConnectivityChecker;

class ConnectivityProcesser : public QObject
{
    Q_OBJECT

public:
    explicit ConnectivityProcesser(NetworkProcesser *networkProcesser, QObject *parent = nullptr);
    ~ConnectivityProcesser();
    Connectivity connectivity() const;
    void setCheckPortal(bool checkPortal);

signals:
    void connectivityChanged(const Connectivity &) const;

private:
    ConnectivityChecker *createConnectivityChecker(bool enableConnectivity);

private slots:
    void onEnableConnectivityChanged(bool enableConnectivity);
    void onConnectivityChanged(const Connectivity &connectivity);

private:
    QScopedPointer<ConnectivityChecker> m_checker;
    NetworkProcesser *m_networkProcesser;
};

}
}

#endif // CONNECTIVITYPROCESSER_H
