// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef IPCONFLICTCHECKER_H
#define IPCONFLICTCHECKER_H

#include "netutils.h"
#include "networkconst.h"

#include <QObject>
#include <QPointer>

namespace dde {
namespace network {

class NetworkDeviceBase;
class NetworkProcesser;
class DeviceIPChecker;

class DeviceIPChecker : public QObject
{
    Q_OBJECT

Q_SIGNALS:
    void conflictStatusChanged(NetworkDeviceBase *, const bool);
    void ipConflictCheck(const QStringList &);

public:
    explicit DeviceIPChecker(NetworkDeviceBase *device, NetworkInter *netInter, QObject *parent);
    ~DeviceIPChecker() override;
    NetworkDeviceBase *device();
    void setDeviceInfo(const QStringList &ipv4, const QString &macAddress);
    void handlerIpConflict();
    QStringList ipV4();
    bool ipConflicted();

private:
    QPointer<NetworkDeviceBase> m_device;
    NetworkInter *m_networkInter;
    QStringList m_ipV4;
    QString m_macAddress;
    int m_conflictCount;
    int m_clearCount;
    int m_count;
    bool m_ipConflicted;
    QList<QStringList> m_changeIpv4s;
    QTime m_requestTime;
};

class IPConflictChecker : public QObject
{
    Q_OBJECT

Q_SIGNALS:
    void conflictStatusChanged(NetworkDeviceBase *, const bool);

public:
    explicit IPConflictChecker(NetworkProcesser *networkProcesser, bool ipChecked);
    ~IPConflictChecker() override;

private Q_SLOT:
    void onDeviceChanged();
    void onIPConflict(const QString &ip, const QString &macAddress);
    void onSenderIPInfo(const QStringList &ips);

private:
    void handlerIpConflict(const QString &ip, const QString &macAddress, const QString &activeConnectionInfo);
    QMap<QString, NetworkDeviceBase *> parseDeviceIp(const QString &activeConnectionInfo);
    void clearUnExistDevice();

private:
    NetworkInter *m_networkInter;
    NetworkProcesser *m_networkProcesser;
    QList<DeviceIPChecker *> m_deviceCheckers;
    bool m_ipNeedCheck;
    QThread *m_thread;
};

}
}

#endif // IPCONFLICTCHECKER_H
